@@ -100,7 +100,6 @@ static void qRegisterMediaPlayerMetaTypes()
 
 Q_CONSTRUCTOR_FUNCTION(qRegisterMediaPlayerMetaTypes)
 
-
 #define MAX_NESTED_PLAYLISTS 16
 
 class QMediaPlayerPrivate : public QMediaObjectPrivate
@@ -112,38 +111,39 @@ public:
         : provider(0)
         , control(0)
         , audioRoleControl(0)
+        , playlist(0)
+        , networkAccessControl(0)
         , state(QMediaPlayer::StoppedState)
         , status(QMediaPlayer::UnknownMediaStatus)
         , error(QMediaPlayer::NoError)
         , ignoreNextStatusChange(-1)
-        , playlist(0)
-        , networkAccessControl(0)
-        , hasStreamPlaybackFeature(false)
         , nestedPlaylists(0)
+        , hasStreamPlaybackFeature(false)
     {}
 
     QMediaServiceProvider *provider;
     QMediaPlayerControl* control;
     QAudioRoleControl *audioRoleControl;
-    QMediaPlayer::State state;
-    QMediaPlayer::MediaStatus status;
-    QMediaPlayer::Error error;
     QString errorString;
-    int ignoreNextStatusChange;
 
     QPointer<QObject> videoOutput;
     QMediaPlaylist *playlist;
     QMediaNetworkAccessControl *networkAccessControl;
     QVideoSurfaceOutput surfaceOutput;
-    bool hasStreamPlaybackFeature;
     QMediaContent qrcMedia;
     QScopedPointer<QFile> qrcFile;
 
     QMediaContent rootMedia;
     QMediaContent pendingPlaylist;
-    QMediaPlaylist *parentPlaylist(QMediaPlaylist *pls);
-    bool isInChain(QUrl url);
+    QMediaPlayer::State state;
+    QMediaPlayer::MediaStatus status;
+    QMediaPlayer::Error error;
+    int ignoreNextStatusChange;
     int nestedPlaylists;
+    bool hasStreamPlaybackFeature;
+
+    QMediaPlaylist *parentPlaylist(QMediaPlaylist *pls);
+    bool isInChain(const QUrl &url);
 
     void setMedia(const QMediaContent &media, QIODevice *stream = 0);
 
@@ -175,7 +175,7 @@ QMediaPlaylist *QMediaPlayerPrivate::parentPlaylist(QMediaPlaylist *pls)
     return 0;
 }
 
-bool QMediaPlayerPrivate::isInChain(QUrl url)
+bool QMediaPlayerPrivate::isInChain(const QUrl &url)
 {
     // Check whether a URL is already in the chain of playlists.
     // Also see a comment in parentPlaylist().
@@ -1372,8 +1372,14 @@ QList<QAudio::Role> QMediaPlayer::supportedAudioRoles() const
     \property QMediaPlayer::volume
     \brief the current playback volume.
 
-    The playback volume is linear in effect and the value can range from 0 -
-    100, values outside this range will be clamped.
+    The playback volume is scaled linearly, ranging from \c 0 (silence) to \c 100 (full volume).
+    Values outside this range will be clamped.
+
+    By default the volume is \c 100.
+
+    UI volume controls should usually be scaled nonlinearly. For example, using a logarithmic scale
+    will produce linear changes in perceived loudness, which is what a user would normally expect
+    from a volume control. See QAudio::convertVolume() for more details.
 */
 
 /*!
